@@ -36,14 +36,21 @@ class BipartiteEventStudyBase(bpd.BipartiteBase):
         '''
         return BipartiteEventStudyBase
 
-    def clean_data(self):
+    def clean_data(self, user_clean={}):
         '''
         Clean data to make sure there are no NaN or duplicate observations, firms are connected by movers and firm ids are contiguous.
 
+        Arguments:
+            user_clean (dict): dictionary of parameters for cleaning
+
+                Dictionary parameters:
+
+                    i_t_how (str): if 'max', keep max paying job; if 'sum', sum over duplicate worker-firm-year observations, then take the highest paying worker-firm sum; if 'mean', average over duplicate worker-firm-year observations, then take the highest paying worker-firm average. Note that if multiple time and/or firm columns are included (as in event study format), then duplicates are cleaned in order of earlier time columns to later time columns, and earlier firm ids to later firm ids
+
         Returns:
             frame (BipartiteEventStudyBase): BipartiteEventStudyBase with cleaned data
         '''
-        frame = bpd.BipartiteBase.clean_data(self)
+        frame = bpd.BipartiteBase.clean_data(self, user_clean)
 
         frame.logger.info('beginning BipartiteEventStudyBase data cleaning')
         frame.logger.info('checking quality of data')
@@ -176,15 +183,31 @@ class BipartiteEventStudyBase(bpd.BipartiteBase):
                 if self.col_dtype_dict[col] == 'int':
                     astype_dict[col] = int
 
+        # Sort by i, t if t included; otherwise sort by i
+        sort_order_1 = ['i']
+        sort_order_2 = ['i']
+        if self.col_included('t'):
+            sort_order_1.append(bpd.to_list(self.reference_dict['t'])[0]) # Pre-reformatting
+            sort_order_2.append(bpd.to_list(self.reference_dict['t'])[0][: - 1]) # Remove last number, e.g. t11 to t1
+
         # Append the last row if a mover (this is because the last observation is only given as an f2i, never as an f1i)
         last_obs_df = pd.DataFrame(self[self['m'] == 1]) \
-            .groupby('i').last() \
-            .rename(rename_dict_1, axis=1) \
-            .reset_index() # Get i out of index, return to column
-        data_long = pd.concat([pd.DataFrame(self), last_obs_df], ignore_index=True) \
-            .drop(drops, axis=1) \
-            .rename(rename_dict_2, axis=1) \
-            .astype(astype_dict) \
+            .sort_values(sort_order_1) \
+            .drop_duplicates(subset='i', keep='last') \
+            .rename(rename_dict_1, axis=1) # Sort by i, t to ensure last observation is actually last
+
+        try:
+            data_long = pd.concat([pd.DataFrame(self), last_obs_df], ignore_index=True) \
+                .drop(drops, axis=1) \
+                .rename(rename_dict_2, axis=1) \
+                .astype(astype_dict)
+        except ValueError: # If nan values, use Int8
+            for col in astype_dict.keys():
+                astype_dict[col] = 'Int64'
+            data_long = pd.concat([pd.DataFrame(self), last_obs_df], ignore_index=True) \
+                .drop(drops, axis=1) \
+                .rename(rename_dict_2, axis=1) \
+                .astype(astype_dict)
         # data_long = pd.DataFrame(self).groupby('i').apply(lambda a: a.append(a.iloc[-1].rename(rename_dict_1, axis=1)) if a.iloc[0]['m'] == 1 else a) \
         #     .reset_index(drop=True) \
         #     .drop(drops, axis=1) \
@@ -193,13 +216,7 @@ class BipartiteEventStudyBase(bpd.BipartiteBase):
 
         # Sort columns and rows
         sorted_cols = sorted(data_long.columns, key=bpd.col_order)
-        try: # Long FIXME try to do this without try/except
-            data_long = data_long[sorted_cols].sort_values(['i', 't']).reset_index(drop=True)
-        except KeyError:
-            try: # Collapsed long
-                data_long = data_long[sorted_cols].sort_values(['i', 't1']).reset_index(drop=True)
-            except KeyError: # No time column
-                data_long = data_long[sorted_cols].sort_values(['i']).reset_index(drop=True)
+        data_long = data_long[sorted_cols].sort_values(sort_order_2).reset_index(drop=True)
 
         if return_df:
             return data_long
